#![% PERL %]
#
# [% service %]: [% description %]
#
# created by: [% author %]
# 
# chkconfig:    345 88 13
# description:  [% description %]
#
### BEGIN INIT INFO
# Provides:          [% provides %]
# Required-Start:
# Required-Stop:
# Should-Start:
# Should-Stop:
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: [% description %]
# Description:       [% description %]
### END INIT INFO

# This script generated by geninit tool

use strict;
use warnings;

use System::InitD::Runner;

# Config
# path to pidfile here:
my $PID             = '[% pid_file %]';
# start command here:
my $START_COMMAND   = "[% start_cmd %]";
# EXACT process name here:
my $PROCESS_NAME    = '[% process_name %]';
my $USAGE           = 'start|stop|usage|status|info';

my $USER            = '[% user %]';

# Run
my $runner = System::InitD::Runner->new(
    usage               =>  $USAGE,
    # proces name here:
    process_name        =>  $PROCESS_NAME,
    # start cmd here
    start               =>  $START_COMMAND,
    restart_timeout     =>  5,
    pid_file            =>  $PID,
    user                =>  $USER,
);

$runner->run();

exit 1;
